<?php

namespace Drupal\cl_editorial\Element;

use Drupal\Component\Utility\Xss;
use Drupal\Core\Extension\ExtensionLifecycle;
use Drupal\Core\Utility\Error;
use Drupal\sdc\Exception\ComponentNotFoundException;
use Drupal\sdc\Plugin\Component;
use Drupal\cl_editorial\NoThemeComponentManager;
use Drupal\Core\File\FileUrlGeneratorInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\Core\Render\Element;
use Drupal\Core\Render\Element\FormElement;
use Drupal\Core\Render\Element\Radios;
use League\CommonMark\CommonMarkConverter;
use League\CommonMark\Exception\CommonMarkException;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Provides a component selector element.
 *
 * @FormElement("cl_component_selector")
 */
class ComponentSelectorElement extends FormElement implements ContainerFactoryPluginInterface {

  protected const DEFAULT_STATUSES = [
    ExtensionLifecycle::STABLE,
    ExtensionLifecycle::EXPERIMENTAL,
  ];

  /**
   * {@inheritdoc}
   */
  public function __construct(
    array $configuration,
    $plugin_id,
    $plugin_definition,
    protected NoThemeComponentManager $componentManager,
    protected FileUrlGeneratorInterface $fileUrlGenerator
  ) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    $component_manager = $container->get(NoThemeComponentManager::class);
    assert($component_manager instanceof NoThemeComponentManager);
    $file_url_generator = $container->get('file_url_generator');
    assert($file_url_generator instanceof FileUrlGeneratorInterface);
    return new static($configuration, $plugin_id, $plugin_definition, $component_manager, $file_url_generator);
  }

  /**
   * @inheritDoc
   */
  public function getInfo() {
    return [
      '#title' => $this->t('Single Directory Components'),
      '#description' => $this->t('Select the suitable SDC component.'),
      '#description_display' => 'before',
      '#filters' => [],
      '#process' => [[$this, 'populateOptions']],
      '#element_validate' => [[$this, 'validateExistingComponent']],
      '#theme_wrappers' => ['cl_component_selector', 'fieldset'],
      '#input' => TRUE,
    ];
  }

  /**
   * Form API process callback.
   */
  public function populateOptions(
    array &$element,
    FormStateInterface $form_state,
    array &$complete_form
  ): array {
    $filters = $element['#filters'] ?? [];
    $components = $this->componentManager->getFilteredComponentTypes(
      $filters['allowed'] ?? [],
      $filters['forbidden'] ?? [],
      $filters['statuses'] ?? static::DEFAULT_STATUSES,
    );

    $options = array_reduce(
      $components,
      static fn(array $carry, Component $component) => [
        ...$carry,
        $component->getPluginId() => $component->metadata->name,
      ],
      []
    );
    ksort($options);

    $element['#attributes']['class'] = ['cl-components--selector'];
    $default_value = $element['#default_value']['machine_name'] ?? NULL;
    $element += [
      '#attached' => ['library' => ['cl_editorial/options-filter']],
      'search' => [
        '#title' => $this->t('Search'),
        '#title_display' => 'hidden',
        '#type' => 'search',
        '#default_value' => $default_value,
        '#placeholder' => $this->t('my-component'),
        '#size' => 30,
        '#description' => $this->t('Start typing the component name or group to reduce the list below.'),
        '#input' => FALSE,
      ],
      'machine_name' => [
        '#type' => 'radios',
        '#options' => $options,
        '#title' => $this->t('Components'),
        '#title_display' => 'hidden',
        '#default_value' => $default_value,
        '#process' => [
          [Radios::class, 'processRadios'],
          [$this, 'processRadios'],
        ],
        '#attributes' => ['class' => ['cl-components-selector--radios']],
        '#input' => FALSE,
        '#ajax' => $element['#ajax'] ?? FALSE,
      ],
    ];
    unset($element['#default_value'], $element['#ajax']);
    return $element;
  }

  public function processRadios(array $element, FormStateInterface $form_state): array {
    unset($element['#value_callback']);
    $keys = Element::children($element);
    foreach ($keys as $key) {
      $element[$key]['#theme_wrappers'] = [
        'form_element__radio__cl_component',
        'form_element__radio',
      ];
      $id = $element[$key]['#return_value'];
      $component = $this->componentManager->findWithoutThemeFilter($id);
      $element[$key]['#component'] = $component;
      $metadata = $component->metadata;
      $element[$key]['#description'] = $metadata->description;
      $element[$key]['#value_callback'] = NULL;
      $element[$key]['#thumbnail_url'] = $metadata->getThumbnailPath() ? $this->fileUrlGenerator->generateAbsoluteString($metadata->getThumbnailPath()) : NULL;
      $element[$key]['#component_status'] = $metadata->status;



      $docs = NULL;
      if (class_exists('League\CommonMark\CommonMarkConverter')) {
        $converter = new CommonMarkConverter();
        try {
          $docs = $converter->convert($metadata->documentation)->getContent();
        }
        catch (CommonMarkException $e) {
          Error::logException($this->getLogger('cl_devel'), $e);
        }
      }
      $docs = $docs ?? nl2br($metadata->documentation);
      $element[$key]['#readme'] = Xss::filterAdmin($docs);
    }
    return $element;
  }

  /**
   * Validator for form element.
   */
  public function validateExistingComponent(array $element, FormStateInterface $form_state): void {
    $parents = $element['#parents'] ?? [];
    $value = $form_state->getValue($parents)['machine_name'] ?? '';
    if (!$value) {
      return;
    }
    try {
      $this->componentManager->findWithoutThemeFilter($value);
    }
    catch (ComponentNotFoundException $e) {
      Error::logException(\Drupal::logger('cl_editorial'), $e);
      $form_state->setError(
        $element['machine_name'],
        $this->t('Invalid component ID: @id', ['@id' => $value])
      );
    }
  }

}
